package Nats

import (
	"Cold_WorkOrder/conf"
	"Cold_WorkOrder/models/Account"
	"fmt"
	"github.com/nats-io/nats.go"
	"github.com/vmihailenco/msgpack/v5"
	"time"
)

var Nats *nats.Conn

func init() {
	fmt.Println("============Nats init============")
	var err error
	// 连接Nats服务器
	Nats, err = nats.Connect("nats://" + conf.NatsServer_Url)
	if err != nil {
		fmt.Println("nats 连接失败！")
		panic(err)
	}
	fmt.Println("nats OK！")

}

// 验证TOKEY
func Verification(ToKey string) (bool, Account.User) {
	// 请求-响应， 向 verification 发布一个 `ToKey` 请求数据，设置超时间3秒，如果有多个响应，只接收第一个收到的消息
	msg, err := Nats.Request("Cold_User_verification", []byte(ToKey), 3*time.Second)
	if err != nil {
		return false, Account.User{}
	}
	fmt.Printf("verification : %s\n", string(msg.Data))
	type T_R struct {
		Code int16        `xml:"Code"`
		Msg  string       `xml:"Msg"`
		Data Account.User `xml:"Data"` // 泛型
	}

	var t_R T_R

	err = msgpack.Unmarshal(msg.Data, &t_R)
	if err != nil {
		return false, Account.User{}
	}

	return true, t_R.Data

}

func CompanyListAllByT_name(T_name string) (list []Account.Company) {
	msg, err := Nats.Request("Cold_User_CompanyListAllByT_name", []byte(T_name), 3*time.Second)
	if err != nil {
		return list
	}
	fmt.Printf("CompanyListAllByT_name: %s\n", string(msg.Data))
	type T_R struct {
		Code int16             `xml:"Code"`
		Msg  string            `xml:"Msg"`
		Data []Account.Company `xml:"Data"` // 泛型
	}
	var t_R T_R

	err = msgpack.Unmarshal(msg.Data, &t_R)
	if err != nil {
		return list
	}

	return t_R.Data
}

func UserListAll() (list []Account.User) {
	msg, err := Nats.Request("Cold_User_UserListAll", []byte(""), 3*time.Second)
	if err != nil {
		return list
	}
	fmt.Printf("UserListAll: %s\n", string(msg.Data))
	type T_R struct {
		Code int16          `xml:"Code"`
		Msg  string         `xml:"Msg"`
		Data []Account.User `xml:"Data"` // 泛型
	}
	var t_R T_R

	err = msgpack.Unmarshal(msg.Data, &t_R)
	if err != nil {
		return list
	}

	return t_R.Data
}

// 添加系统日志
func AddSysLogs(T_class, T_title string, T_txt interface{}) {

	type T_S struct {
		T_class string
		T_title string
		T_txt   interface{}
	}
	b, _ := msgpack.Marshal(&T_S{T_class: T_class, T_title: T_title, T_txt: T_txt})
	// 发布-订阅 模式，向 test1 发布一个 `Hello World` 数据
	_ = Nats.Publish("Cold_User_AddSysLogs", b)

}

// 添加用户日志
func AddUserLogs(T_uuid, T_class, T_title string, T_txt interface{}) {

	type T_S struct {
		T_uuid  string
		T_class string
		T_title string
		T_txt   interface{}
	}
	b, _ := msgpack.Marshal(&T_S{T_uuid: T_uuid, T_class: T_class, T_title: T_title, T_txt: T_txt})
	// 发布-订阅 模式，向 test1 发布一个 `Hello World` 数据
	_ = Nats.Publish("Cold_User_AddUserLogs", b)
}
